------------------------------------------------------------------------------
--                                                                          --
--                         GNAT RUN-TIME COMPONENTS                         --
--                                                                          --
--             G N A T . T R A C E B A C K . S Y M B O L I C                --
--                                                                          --
--                                 B o d y                                  --
--                                                                          --
--                     Copyright (C) 1999-2005, AdaCore                     --
--                                                                          --
-- GNAT is free software;  you can  redistribute it  and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion.  GNAT is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License  distributed with GNAT;  see file COPYING.  If not, write --
-- to  the  Free Software Foundation,  51  Franklin  Street,  Fifth  Floor, --
-- Boston, MA 02110-1301, USA.                                              --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable  to  be --
-- covered  by the  GNU  General  Public  License.  This exception does not --
-- however invalidate  any other reasons why  the executable file  might be --
-- covered by the  GNU Public License.                                      --
--                                                                          --
-- GNAT was originally developed  by the GNAT team at  New York University. --
-- Extensive contributions were provided by Ada Core Technologies Inc.      --
--                                                                          --
------------------------------------------------------------------------------

--  Run-time symbolic traceback support
--  This file has been modified by Juergen Pfeifer (31-Dec-1999) for
--  the purpose to support the Ada for Linux Team implementation of
--  convert_addresses. This implementation has the advantage to run
--  on the binutils as they are deployed on Linux.

with System.Soft_Links;
with Ada.Exceptions.Traceback; use Ada.Exceptions.Traceback;

package body GNAT.Traceback.Symbolic is

   package TSL renames System.Soft_Links;

   ------------------------
   -- Symbolic_Traceback --
   ------------------------

   function Symbolic_Traceback (Traceback : Tracebacks_Array) return String is
      procedure convert_addresses
        (addrs  : System.Address;
         n_addr : Integer;
         buf    : System.Address;
         len    : System.Address);
      pragma Import (C, convert_addresses, "convert_addresses");
      --  This is the ALT Linux specific version adapted to the binutils
      --  deployed with most Linuxes. These binutils already have an
      --  addr2line tool that demangles Ada symbolic names, but there are
      --  version dependant variants for the option names. Therefore our
      --  implementation spawns a shell script that does normalization of
      --  the option and then executes addr2line and communicates with it
      --  through a bidirectional pipe.
      --
      --  Note that this procedure is *not* thread-safe.

      B_Size : constant Integer := 256 * Traceback'Length;
      Len    : Integer := B_Size;
      Res    : String (1 .. B_Size);

   begin
      if Traceback'Length > 0 then
         TSL.Lock_Task.all;
         convert_addresses
           (Traceback'Address, Traceback'Length, Res (1)'Address, Len'Address);
         TSL.Unlock_Task.all;
         return Res (1 .. Len);
      else
         return "";
      end if;
   end Symbolic_Traceback;

   function Symbolic_Traceback (E : Exception_Occurrence) return String is
   begin
      return Symbolic_Traceback (Tracebacks (E));
   end Symbolic_Traceback;

end GNAT.Traceback.Symbolic;
